@@ -1,5 +1,5 @@
 # Copyright 2003-2008 by Leighton Pritchard.  All rights reserved.
-# Revisions copyright 2008-2012 by Peter Cock.
+# Revisions copyright 2008-2017 by Peter Cock.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -16,7 +16,6 @@ from __future__ import print_function
 
 from reportlab.graphics.shapes import Drawing, String, Group, Line, Circle, Polygon
 from reportlab.lib import colors
-from reportlab.pdfbase import _fontdata
 from reportlab.graphics.shapes import ArcPath
 
 from Bio._py3k import range
@@ -27,185 +26,84 @@ from ._AbstractDrawer import _stroke_and_fill_colors
 from ._FeatureSet import FeatureSet
 from ._GraphSet import GraphSet
 
-from math import ceil, pi, cos, sin, asin
+from math import pi, cos, sin
 
 
 class CircularDrawer(AbstractDrawer):
     """Object for drawing circular diagrams.
 
-        o __init__(self, ...) Called on instantiation
+    Attributes:
+     - tracklines    Boolean for whether to draw lines dilineating tracks
+     - pagesize      Tuple describing the size of the page in pixels
+     - x0            Float X co-ord for leftmost point of drawable area
+     - xlim          Float X co-ord for rightmost point of drawable area
+     - y0            Float Y co-ord for lowest point of drawable area
+     - ylim          Float Y co-ord for topmost point of drawable area
+     - pagewidth     Float pixel width of drawable area
+     - pageheight    Float pixel height of drawable area
+     - xcenter       Float X co-ord of center of drawable area
+     - ycenter       Float Y co-ord of center of drawable area
+     - start         Int, base to start drawing from
+     - end           Int, base to stop drawing at
+     - length        Size of sequence to be drawn
+     - track_size    Float (0->1) the proportion of the track height to draw in
+     - drawing       Drawing canvas
+     - drawn_tracks  List of ints denoting which tracks are to be drawn
+     - current_track_level   Int denoting which track is currently being drawn
+     - track_offsets     Dictionary of number of pixels that each track top,
+       center and bottom is offset from the base of a fragment, keyed by track
+     - sweep     Float (0->1) the proportion of the circle circumference to
+       use for the diagram
+     - cross_track_links List of tuples each with four entries (track A,
+       feature A, track B, feature B) to be linked.
 
-        o set_page_size(self, pagesize, orientation)    Set the page size to the
-                                                    passed size and orientation
-
-        o set_margins(self, x, y, xl, xr, yt, yb)   Set the drawable area of the
-                                                    page
-
-        o set_bounds(self, start, end)  Set the bounds for the elements to be
-                                        drawn
-
-        o is_in_bounds(self, value)     Returns a boolean for whether the position
-                                        is actually to be drawn
-
-        o __len__(self)     Returns the length of sequence that will be drawn
-
-
-        o draw(self)    Place the drawing elements on the diagram
-
-        o init_fragments(self)  Calculate information
-                                about sequence fragment locations on the drawing
-
-        o set_track_heights(self)   Calculate information about the offset of
-                                    each track from the fragment base
-
-        o draw_test_tracks(self)    Add lines demarcating each track to the
-                                    drawing
-
-        o draw_track(self, track)   Return the contents of the passed track as
-                                    drawing elements
-
-        o draw_scale(self, track)   Return a scale for the passed track as
-                                    drawing elements
-
-        o draw_greytrack(self, track)   Return a grey background and superposed
-                                        label for the passed track as drawing
-                                        elements
-
-        o draw_feature_set(self, set)   Return the features in the passed set as
-                                        drawing elements
-
-        o draw_feature(self, feature)   Return a single feature as drawing
-                                        elements
-
-        o get_feature_sigil(self, feature, x0, x1, fragment)    Return a single
-                                        feature as its sigil in drawing elements
-
-        o draw_graph_set(self, set)     Return the data in a set of graphs as
-                                        drawing elements
-
-        o draw_line_graph(self, graph)  Return the data in a graph as a line
-                                        graph in drawing elements
-
-        o draw_heat_graph(self, graph)  Return the data in a graph as a heat
-                                        graph in drawing elements
-
-        o draw_bar_graph(self, graph)   Return the data in a graph as a bar
-                                        graph in drawing elements
-
-        o canvas_angle(self, base)      Return the angle, and cos and sin of
-                                        that angle, subtended by the passed
-                                        base position at the diagram center
-
-        o draw_arc(self, inner_radius, outer_radius, startangle, endangle,
-                    color)    Return a drawable element describing an arc
-
-        Attributes:
-
-        o tracklines    Boolean for whether to draw lines dilineating tracks
-
-        o pagesize      Tuple describing the size of the page in pixels
-
-        o x0            Float X co-ord for leftmost point of drawable area
-
-        o xlim          Float X co-ord for rightmost point of drawable area
-
-        o y0            Float Y co-ord for lowest point of drawable area
-
-        o ylim          Float Y co-ord for topmost point of drawable area
-
-        o pagewidth     Float pixel width of drawable area
-
-        o pageheight    Float pixel height of drawable area
-
-        o xcenter       Float X co-ord of center of drawable area
-
-        o ycenter       Float Y co-ord of center of drawable area
-
-        o start         Int, base to start drawing from
-
-        o end           Int, base to stop drawing at
-
-        o length        Size of sequence to be drawn
-
-        o track_size    Float (0->1) the proportion of the track height to
-                        draw in
-
-        o drawing       Drawing canvas
-
-        o drawn_tracks  List of ints denoting which tracks are to be drawn
-
-        o current_track_level   Int denoting which track is currently being
-                                drawn
-
-        o track_offsets     Dictionary of number of pixels that each track top,
-                            center and bottom is offset from the base of a
-                            fragment, keyed by track
-
-        o sweep     Float (0->1) the proportion of the circle circumference to
-                    use for the diagram
-
-        o cross_track_links List of tuples each with four entries (track A,
-                            feature A, track B, feature B) to be linked.
     """
+
     def __init__(self, parent=None, pagesize='A3', orientation='landscape',
                  x=0.05, y=0.05, xl=None, xr=None, yt=None, yb=None,
                  start=None, end=None, tracklines=0, track_size=0.75,
                  circular=1, circle_core=0.0, cross_track_links=None):
         """Create CircularDrawer object.
 
-            o parent    Diagram object containing the data that the drawer
-                        draws
+        Arguments:
+         - parent    Diagram object containing the data that the drawer
+           draws
+         - pagesize  String describing the ISO size of the image, or a tuple
+           of pixels
+         - orientation   String describing the required orientation of the
+           final drawing ('landscape' or 'portrait')
+         - x         Float (0->1) describing the relative size of the X
+           margins to the page
+         - y         Float (0->1) describing the relative size of the Y
+           margins to the page
+         - xl        Float (0->1) describing the relative size of the left X
+           margin to the page (overrides x)
+         - xl        Float (0->1) describing the relative size of the left X
+           margin to the page (overrides x)
+         - xr        Float (0->1) describing the relative size of the right X
+           margin to the page (overrides x)
+         - yt        Float (0->1) describing the relative size of the top Y
+           margin to the page (overrides y)
+         - yb        Float (0->1) describing the relative size of the lower Y
+           margin to the page (overrides y)
+         - start     Int, the position to begin drawing the diagram at
+         - end       Int, the position to stop drawing the diagram at
+         - tracklines    Boolean flag to show (or not) lines delineating tracks
+           on the diagram
+         - track_size    The proportion of the available track height that
+           should be taken up in drawing
+         - circular      Boolean flaw to show whether the passed sequence is
+           circular or not
+         - circle_core   The proportion of the available radius to leave
+           empty at the center of a circular diagram (0 to 1).
+         - cross_track_links List of tuples each with four entries (track A,
+           feature A, track B, feature B) to be linked.
 
-            o pagesize  String describing the ISO size of the image, or a tuple
-                        of pixels
-
-            o orientation   String describing the required orientation of the
-                            final drawing ('landscape' or 'portrait')
-
-            o x         Float (0->1) describing the relative size of the X
-                        margins to the page
-
-            o y         Float (0->1) describing the relative size of the Y
-                        margins to the page
-
-            o xl        Float (0->1) describing the relative size of the left X
-                        margin to the page (overrides x)
-
-            o xl        Float (0->1) describing the relative size of the left X
-                        margin to the page (overrides x)
-
-            o xr        Float (0->1) describing the relative size of the right X
-                        margin to the page (overrides x)
-
-            o yt        Float (0->1) describing the relative size of the top Y
-                        margin to the page (overrides y)
-
-            o yb        Float (0->1) describing the relative size of the lower Y
-                        margin to the page (overrides y)
-
-            o start     Int, the position to begin drawing the diagram at
-
-            o end       Int, the position to stop drawing the diagram at
-
-            o tracklines    Boolean flag to show (or not) lines delineating tracks
-                            on the diagram
-
-            o track_size    The proportion of the available track height that
-                            should be taken up in drawing
-
-            o circular      Boolean flaw to show whether the passed sequence is
-                            circular or not
-
-            o circle_core   The proportion of the available radius to leave
-                            empty at the center of a circular diagram (0 to 1).
-
-            o cross_track_links List of tuples each with four entries (track A,
-                                feature A, track B, feature B) to be linked.
         """
         # Use the superclass' instantiation method
         AbstractDrawer.__init__(self, parent, pagesize, orientation,
-                                  x, y, xl, xr, yt, yb, start, end,
-                                  tracklines, cross_track_links)
+                                x, y, xl, xr, yt, yb, start, end,
+                                tracklines, cross_track_links)
 
         # Useful measurements on the page
         self.track_size = track_size
@@ -216,7 +114,7 @@ class CircularDrawer(AbstractDrawer):
             self.sweep = 1
 
     def set_track_heights(self):
-        """Initialise track heights.
+        """Initialize track heights.
 
         Since tracks may not be of identical heights, the bottom and top
         radius for each track is stored in a dictionary - self.track_radii,
@@ -231,7 +129,7 @@ class CircularDrawer(AbstractDrawer):
         for track in range(bot_track, top_track + 1):  # track numbers to 'draw'
             try:
                 trackheight = self._parent[track].height  # Get track height
-            except:
+            except Exception:  # TODO: ValueError? IndexError?
                 trackheight = 1
             trackunit_sum += trackheight  # increment total track unit height
             trackunits[track] = (heightholder, heightholder + trackheight)
@@ -310,7 +208,7 @@ class CircularDrawer(AbstractDrawer):
             self.draw_test_tracks()
 
     def draw_track(self, track):
-        """Returns list of track elements and list of track labels."""
+        """Return list of track elements and list of track labels."""
         track_elements = []  # Holds elements for features and graphs
         track_labels = []   # Holds labels for features and graphs
 
@@ -326,7 +224,7 @@ class CircularDrawer(AbstractDrawer):
         return track_elements, track_labels
 
     def draw_feature_set(self, set):
-        """Returns list of feature elements and list of labels for them."""
+        """Return list of feature elements and list of labels for them."""
         # print 'draw feature set'
         feature_elements = []  # Holds diagram elements belonging to the features
         label_elements = []   # Holds diagram elements belonging to feature labels
@@ -341,7 +239,7 @@ class CircularDrawer(AbstractDrawer):
         return feature_elements, label_elements
 
     def draw_feature(self, feature):
-        """Returns list of feature elements and list of labels for them."""
+        """Return list of feature elements and list of labels for them."""
         feature_elements = []  # Holds drawable elements for a single feature
         label_elements = []   # Holds labels for a single feature
 
@@ -366,13 +264,13 @@ class CircularDrawer(AbstractDrawer):
         return feature_elements, label_elements
 
     def get_feature_sigil(self, feature, locstart, locend, **kwargs):
-        """Returns graphics for feature, and any required label for it.
+        """Return graphics for feature, and any required label for it.
 
-            o feature       Feature object
+        Arguments:
+         - feature       Feature object
+         - locstart      The start position of the feature
+         - locend        The end position of the feature
 
-            o locstart      The start position of the feature
-
-            o locend        The end position of the feature
         """
         # Establish the co-ordinates for the sigil
         btm, ctr, top = self.track_radii[self.current_track_level]
@@ -475,6 +373,7 @@ class CircularDrawer(AbstractDrawer):
         return sigil, labelgroup
 
     def draw_cross_link(self, cross_link):
+        """Draw a cross-link between features."""
         startA = cross_link.startA
         startB = cross_link.startB
         endA = cross_link.endA
@@ -549,9 +448,11 @@ class CircularDrawer(AbstractDrawer):
                            cross_link.color, cross_link.border, cross_link.flip)]
 
     def draw_graph_set(self, set):
-        """Returns list of graph elements and list of their labels.
+        """Return list of graph elements and list of their labels.
+
+        Arguments:
+         - set       GraphSet object
 
-            o set       GraphSet object
         """
         # print 'draw graph set'
         elements = []  # Holds graph elements
@@ -568,9 +469,11 @@ class CircularDrawer(AbstractDrawer):
         return elements, []
 
     def draw_line_graph(self, graph):
-        """Returns line graph as list of drawable elements.
+        """Return line graph as list of drawable elements.
+
+        Arguments:
+         - graph     GraphData object
 
-            o graph     GraphData object
         """
         line_elements = []  # holds drawable elements
 
@@ -619,9 +522,11 @@ class CircularDrawer(AbstractDrawer):
         return line_elements
 
     def draw_bar_graph(self, graph):
-        """Returns list of drawable elements for a bar graph.
+        """Return list of drawable elements for a bar graph.
+
+        Arguments:
+         - graph     Graph object
 
-            o graph     Graph object
         """
         # At each point contained in the graph data, we draw a vertical bar
         # from the track center to the height of the datapoint value (positive
@@ -678,9 +583,11 @@ class CircularDrawer(AbstractDrawer):
         return bar_elements
 
     def draw_heat_graph(self, graph):
-        """Returns list of drawable elements for the heat graph.
+        """Return list of drawable elements for the heat graph.
+
+        Arguments:
+         - graph     Graph object
 
-            o graph     Graph object
         """
         # At each point contained in the graph data, we draw a box that is the
         # full height of the track, extending from the midpoint between the
@@ -717,9 +624,11 @@ class CircularDrawer(AbstractDrawer):
         return heat_elements
 
     def draw_scale(self, track):
-        """Returns list of elements in the scale and list of their labels.
+        """Return list of elements in the scale and list of their labels.
+
+        Arguments:
+         - track     Track object
 
-            o track     Track object
         """
         scale_elements = []  # holds axes and ticks
         scale_labels = []  # holds labels
@@ -872,17 +781,15 @@ class CircularDrawer(AbstractDrawer):
         return scale_elements, scale_labels
 
     def draw_tick(self, tickpos, ctr, ticklen, track, draw_label):
-        """Returns drawing element for a tick on the scale.
+        """Return drawing element for a tick on the scale.
 
-            o tickpos   Int, position of the tick on the sequence
+        Arguments:
+         - tickpos   Int, position of the tick on the sequence
+         - ctr       Float, Y co-ord of the center of the track
+         - ticklen   How long to draw the tick
+         - track     Track, the track the tick is drawn on
+         - draw_label    Boolean, write the tick label?
 
-            o ctr       Float, Y co-ord of the center of the track
-
-            o ticklen   How long to draw the tick
-
-            o track     Track, the track the tick is drawn on
-
-            o draw_label    Boolean, write the tick label?
         """
         # Calculate tick co-ordinates
         tickangle, tickcos, ticksin = self.canvas_angle(tickpos)
@@ -937,10 +844,7 @@ class CircularDrawer(AbstractDrawer):
                                     fillColor=None))  # bottom line
 
     def draw_greytrack(self, track):
-        """Drawing element for grey background to passed track.
-
-            o track     Track object
-        """
+        """Drawing element for grey background to passed Track object."""
         greytrack_bgs = []  # Holds track backgrounds
         greytrack_labels = []  # Holds track foreground labels
 
@@ -1018,20 +922,17 @@ class CircularDrawer(AbstractDrawer):
 
     def _draw_arc(self, inner_radius, outer_radius, startangle, endangle,
                  color, border=None, colour=None, **kwargs):
-        """Returns close path describing an arc box.
+        """Return closed path describing an arc box.
 
-            o inner_radius  Float distance of inside of arc from drawing center
-
-            o outer_radius  Float distance of outside of arc from drawing center
-
-            o startangle    Float angle subtended by start of arc at drawing center
-                            (in radians)
-
-            o endangle      Float angle subtended by end of arc at drawing center
-                            (in radians)
-
-            o color        colors.Color object for arc (overridden by backwards
-                           compatible argument with UK spelling, colour).
+        Arguments:
+         - inner_radius  Float distance of inside of arc from drawing center
+         - outer_radius  Float distance of outside of arc from drawing center
+         - startangle    Float angle subtended by start of arc at drawing center
+           (in radians)
+         - endangle      Float angle subtended by end of arc at drawing center
+           (in radians)
+         - color        colors.Color object for arc (overridden by backwards
+           compatible argument with UK spelling, colour).
 
         Returns a closed path object describing an arced box corresponding to
         the passed values.  For very small angles, a simple four sided
@@ -1074,7 +975,7 @@ class CircularDrawer(AbstractDrawer):
 
     def _draw_arc_line(self, path, start_radius, end_radius, start_angle, end_angle,
                        move=False):
-        """Adds a list of points to a path object.
+        """Add a list of points to a path object.
 
         Assumes angles given are in degrees!
 
@@ -1105,7 +1006,7 @@ class CircularDrawer(AbstractDrawer):
                        outer_startangle, outer_endangle,
                        color, border=None, flip=False,
                        **kwargs):
-        """Returns polygon path describing an arc."""
+        """Return polygon path describing an arc."""
         strokecolor, color = _stroke_and_fill_colors(color, border)
 
         x0, y0 = self.xcenter, self.ycenter      # origin of the circle
